package com.hzya.frame.plugin.kjs.listener;

import com.alibaba.fastjson.JSONObject;
import com.dingtalk.open.app.api.GenericEventListener;
import com.dingtalk.open.app.api.OpenDingTalkStreamClientBuilder;
import com.dingtalk.open.app.api.message.GenericOpenDingTalkEvent;
import com.dingtalk.open.app.api.security.AuthClientCredential;
import com.dingtalk.open.app.stream.protocol.event.EventAckStatus;
import com.hzya.frame.dingtalk.service.IDingTalkExtService;
import com.hzya.frame.plugin.kjs.service.GenericEventConsumer;
import com.hzya.frame.sysnew.application.dao.ISysApplicationDao;
import com.hzya.frame.sysnew.application.entity.SysApplicationEntity;
import com.hzya.frame.sysnew.application.service.ISysApplicationService;
import com.hzya.frame.sysnew.application.service.impl.ApplicationCache;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.DependsOn;
import org.springframework.core.task.TaskExecutor;
import org.springframework.stereotype.Component;
import javax.annotation.PostConstruct;
import javax.annotation.Resource;

/**
 * @Description 钉钉通讯录事件订阅
 * @Author xiangerlin
 * @Date 2024/8/27 14:43
 **/
@Component
public class GenericEventSubscribe {

     Logger logger = LoggerFactory.getLogger(getClass());
    @Resource
    private GenericEventConsumer genericEventConsumer;
    @Resource
    private TaskExecutor taskExecutor;
    @Resource
    private ApplicationCache applicationCache;
    @Resource
    private IDingTalkExtService dingTalkExtService;
    @Resource
    private ISysApplicationDao sysApplicationDao;
    @PostConstruct
    public void init() {
        //查询钉钉配置的参数
        SysApplicationEntity sysApp = new SysApplicationEntity();
        sysApp.setAppId(800005);
        sysApp = sysApplicationDao.queryOne(sysApp);
        if (null != sysApp){
            JSONObject dingTalkConfig = dingTalkExtService.getDingTalkConfig(sysApp);
            if (null != dingTalkConfig){
                String appKey = dingTalkConfig.getString("appKey");
                String appSecret = dingTalkConfig.getString("appSecret");
                subscribe(appKey,appSecret);
            }
        }
    }
    /**
     * 通讯录事件订阅
     */
    public void subscribe(String appKey,String appSecret){
        try {
            logger.info("通讯录事件订阅");
            OpenDingTalkStreamClientBuilder
                    .custom()
                    .credential(new AuthClientCredential(appKey, appSecret))
                    //注册事件监听
                    .registerAllEventListener(new GenericEventListener() {
                        public EventAckStatus onEvent(GenericOpenDingTalkEvent event) {
                            try {
                                try {
                                    //处理事件
                                    taskExecutor.execute(()->{
                                        genericEventConsumer.consume(event);
                                    });
                                }catch (Exception e){
                                    logger.error("GenericEventThreadPool错误：{}",e);
                                }
                                //消费成功
                                return EventAckStatus.SUCCESS;
                            } catch (Exception e) {
                                //消费失败
                                return EventAckStatus.LATER;
                            }
                        }
                    })
                    .build().start();
        }catch (Exception e){
            e.printStackTrace();
            logger.error("通讯录事件订阅出错:{}",e);
        }
    }
}
