package com.hzya.frame.plugin.kjs.service;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.map.MapBuilder;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.http.HttpRequest;
import com.dingtalk.api.response.OapiV2DepartmentGetResponse;
import com.dingtalk.api.response.OapiV2UserGetResponse;
import com.dingtalk.open.app.api.message.GenericOpenDingTalkEvent;
import com.hzya.frame.dingtalk.enums.OrgEventEnum;
import com.hzya.frame.dingtalk.service.IDingTalkService;
import com.hzya.frame.uuid.UUIDUtils;
import com.hzya.frame.web.entity.BaseResult;
import com.hzya.frame.web.entity.JsonResultEntity;
import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;
import shade.com.alibaba.fastjson2.JSONArray;
import shade.com.alibaba.fastjson2.JSONObject;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Description 通讯录事件订阅消费者
 * @Author xiangerlin
 * @Date 2024/8/27 15:33
 **/
public class GenericEventConsumer {
    Logger logger = LoggerFactory.getLogger(getClass());
    //中台应用
    public static final String APPCODE = "800004";
    public static final String URL = "http://127.0.0.1:9999/kangarooDataCenterV3/entranceController/externalCallInterface";
    //中台 publicKey
    public static final String PUBLICKEY = "ZJYAWb7lhAUTYqekPkU+uHJv1/ObJxb7dT7sD8HPRDGAgyhCe7eDIk+3zDUT+v578prj";
    //中台secretKey
    public static final String SECRETKEY = "fviZnLBsQUAGF8w8FSOdJi7XlIm/XAZclMxRagDLfTyJFlvnIBF3w66Hrpfzs8cYj3JzOP8MtA1LSGvL+2BWG8c/o7DKi92S4mr3zcGearA=";
    //mdm通用新增接口编码
    public static final String MDMADDCODE = "8000040022";
    //mdm通用删除接口编码
    public static final String MDMREMOVECODE = "8000040024";
    //主数据人员钉钉档案编码
    public static final String DDUSERMDMCODE = "10015";
    //主数据钉钉部门档案编码
    public static final String DDDEPTMDMCODE = "10016";
    //钉钉应用编码
    public static final String DDAPPCODE = "800005";
    //钉钉 publicKey
    public static final String DDPUBLICKEY = "ZJYAn/EBWEhLUMezDLU4iZ1vTO9kc6pM6XrYLajnqnK60Q9Ce7eDIk+3zDUT+v578prj";
    //钉钉secretKey
    public static final String DDSECRETKEY = "ctMIYyauwoKSFeU4tg5gH1aWC/3OJK6HsKJrSR0oyDmdmdvRNgdoTzX0C1OQ+whrj3JzOP8MtA1LSGvL+2BWG8c/o7DKi92S4mr3zcGearA=";
    //获取钉钉用户详情接口编码
    public static final String GETUSERAPPCODE="8000050001";
    //获取钉钉部门详情接口编码
    public static final String GETDETPAPPCODE="8000050002";
    /**
     * 消费通讯录事件订阅消息
     *
     * @param event
     */
    public void consume(GenericOpenDingTalkEvent event) {
        //事件唯一Id
        String eventId = event.getEventId();
        //事件类型
        String eventType = event.getEventType();
        //事件产生时间
        Long bornTime = event.getEventBornTime();
        //获取事件体
        JSONObject bizData = event.getData();
        logger.info("事件唯一id{},事件类型:{},参数:{}",eventId,eventType,JSONObject.toJSONString(bizData));
        OrgEventEnum orgEventEnum = OrgEventEnum.getByCode(eventType);
        String apiCode = MDMADDCODE;
        String type = "user";
        boolean flag = false;
        switch (orgEventEnum){
            //新增到主数据
            case USER_ADD_ORG:
            case USER_LEAVE_ORG:
            case USER_MODIFY_ORG:
                flag = true;
                break;
            case ORG_DEPT_CREATE:
            case ORG_DEPT_MODIFY:
                type = "dept";
                flag = true;
                break;
            case ORG_DEPT_REMOVE:
                type = "dept";
                apiCode = MDMREMOVECODE;
                flag = true;
                break;
        }
        if (flag){
            if ("user".equals(type)){
                JSONArray userIds = bizData.getJSONArray("userId");
                //调用钉钉查询用户详情接口
                for (int i=0; i<userIds.size();i++){
                    //获取用户详情
                    String userRes = getUserById(String.valueOf(userIds.get(i)));
                    //保存钉钉用户到mdm
                    String userMdmRes = createUser(userRes,eventType);
                }
            }else if ("dept".equals(type)){
                JSONArray deptIdArr = bizData.getJSONArray("deptId");
                for (int i=0; i<deptIdArr.size(); i++){
                    //查询部门详情
                    String deptRes = getDeptById(deptIdArr.getLong(i));
                    //保存部门到mdm
                    String deptMdmRes = createDept(deptRes,apiCode);
                }
            }
        }
    }

    /**
     * 查询钉钉用户详情
     * @param userid
     * @return
     */
    private static String getUserById(String userid) {
        Map<String, String> userHeaderMap = MapBuilder.<String, String>create(true)
                .put("apiCode", GETUSERAPPCODE)//钉钉接口
                .put("publicKey",PUBLICKEY)//中台应用
                .put("secretKey",SECRETKEY)//中台应用
                .put("appId",DDAPPCODE)//钉钉接口
                .build();
        Map<String,Object> userParam = new HashMap<>();
        userParam.put("userid", userid);
        userParam.put("language","zh_CN");
        String userRes = HttpRequest.post(URL).addHeaders(userHeaderMap).body(com.alibaba.fastjson.JSONObject.toJSONString(userParam)).timeout(60000).execute().body();
        return userRes;
    }

    /**
     * 查询钉钉部门详情
     * @param deptId
     * @return
     */
    private String getDeptById(Long deptId) {
        Map<String, String> userHeaderMap = MapBuilder.<String, String>create(true)
                .put("apiCode", GETDETPAPPCODE)//钉钉接口
                .put("publicKey",PUBLICKEY)//中台应用
                .put("secretKey",SECRETKEY)//中台应用
                .put("appId",DDAPPCODE)//钉钉接口
                .build();
        Map<String,Object> deptParam = new LinkedHashMap<>();
        deptParam.put("dept_id",deptId);
        deptParam.put("language","zh_CN");
        String deptRes = HttpRequest.post(URL).addHeaders(userHeaderMap).body(com.alibaba.fastjson.JSONObject.toJSONString(deptParam)).timeout(60000).execute().body();
        return deptRes;
    }

    //保存用户到mdm
    private  String createUser(String userRes,String eventType) {
        JsonResultEntity jsonResult = JSONObject.parseObject(userRes,JsonResultEntity.class);
        if (jsonResult.isFlag()){
            shade.com.alibaba.fastjson2.JSONObject attribute = (shade.com.alibaba.fastjson2.JSONObject) jsonResult.getAttribute();
            String attrStr = attribute.toString();
            OapiV2UserGetResponse rsp = JSONObject.parseObject(attrStr, OapiV2UserGetResponse.class);
            if (rsp.isSuccess()){
                String result = attribute.getString("result");
                OapiV2UserGetResponse.UserGetResponse userInfo = com.alibaba.fastjson.JSONObject.parseObject(result,OapiV2UserGetResponse.UserGetResponse.class);
                //OapiV2UserGetResponse.UserGetResponse userInfo = dingtalkService.getUserById(userIds.getString(i));
                createMdmUser(eventType, userInfo);
                // return body;
            }
        }
        return null;
    }

    public  void createMdmUser(String eventType, OapiV2UserGetResponse.UserGetResponse userInfo) {
        Map<String,Object> mdmMap = new LinkedHashMap<>();
        mdmMap.put("mdmCode",DDUSERMDMCODE);
        mdmMap.put("optionName","系统管理员");
        Map<String,Object> userMap = new LinkedHashMap<>();
        userMap.put("data_id", userInfo.getUserid());
        userMap.put("unionid", userInfo.getUnionid());
        userMap.put("userid", userInfo.getUserid());
        userMap.put("name", userInfo.getName());
        userMap.put("mobile", userInfo.getMobile());
        userMap.put("job_number", userInfo.getJobNumber());
        userMap.put("title", userInfo.getTitle());
        userMap.put("email", userInfo.getEmail());
        userMap.put("org_email", userInfo.getOrgEmail());
        userMap.put("event_type", eventType);
        userMap.put("dept_id_list", userInfo.getDeptIdList());
        if (null != userInfo.getHiredDate()){
            userMap.put("hired_date", DateUtil.format(DateUtil.date(userInfo.getHiredDate()),"yyyy-MM-dd"));
        }
        List<Map<String,Object>> postList = new LinkedList<>();
        //处理职位信息
        List<OapiV2UserGetResponse.DeptPosition> deptPositionList = userInfo.getDeptPositionList();
        if (CollectionUtils.isNotEmpty(deptPositionList) && deptPositionList.size()>1){
            //用stream方式给deptOrderList排序，排序字段为order
            List<OapiV2UserGetResponse.DeptOrder> deptOrderList = userInfo.getDeptOrderList();
            // 使用Stream API对deptOrderList按order字段排序
            List<OapiV2UserGetResponse.DeptOrder> sortedDeptOrderList = deptOrderList.stream()
                    .sorted(Comparator.comparing(OapiV2UserGetResponse.DeptOrder::getOrder))
                    .collect(Collectors.toList());
            for (OapiV2UserGetResponse.DeptOrder deptOrder : sortedDeptOrderList) {
                for (OapiV2UserGetResponse.DeptPosition deptPosition : deptPositionList) {
                    if (deptPosition.getDeptId().equals(deptOrder.getDeptId())){
                        Map<String,Object> positionMap = new LinkedHashMap<>();
                        positionMap.put("dept_id",deptPosition.getDeptId());
                        positionMap.put("title",deptPosition.getTitle());
                        postList.add(positionMap);
                        break;
                    }
                }
            }
        }else if (CollectionUtils.isNotEmpty(deptPositionList) && deptPositionList.size() == 1){
            OapiV2UserGetResponse.DeptPosition deptPosition = deptPositionList.get(0);
            Map<String,Object> positionMap = new LinkedHashMap<>();
            positionMap.put("dept_id",deptPosition.getDeptId());
            positionMap.put("title",StrUtil.isEmpty(deptPosition.getTitle()) ? userInfo.getTitle():deptPosition.getTitle());
            positionMap.put("data_id", UUIDUtils.getUUID());
            postList.add(positionMap);
        }
        mdmMap.put("mdm_dd_user",userMap);
        mdmMap.put("mdm_dd_user_position",postList);
        //保存到人员主数据
        Map<String, String> headerMap = MapBuilder.<String, String>create(true)
                .put("apiCode", MDMADDCODE)//中台接口
                .put("publicKey",DDPUBLICKEY)//钉钉应用
                .put("secretKey",DDSECRETKEY)//钉钉应用
                .put("appId",APPCODE)//中台应用
                .build();
        String body = HttpRequest.post(URL).addHeaders(headerMap).body(com.alibaba.fastjson.JSONObject.toJSONString(mdmMap)).timeout(60000).execute().body();
    }

    /**
     * 保存部门到mdm
     * @param deptRes
     * @return
     */
    private String createDept(String deptRes,String apiCode) {
        if (StrUtil.isNotEmpty(deptRes) && StrUtil.isNotEmpty(apiCode)){
            JsonResultEntity jsonResult = JSONObject.parseObject(deptRes,JsonResultEntity.class);
            if (jsonResult.isFlag()){
                shade.com.alibaba.fastjson2.JSONObject attribute = (shade.com.alibaba.fastjson2.JSONObject) jsonResult.getAttribute();
                String result = attribute.getString("result");
                OapiV2DepartmentGetResponse.DeptGetResponse deptInfo = com.alibaba.fastjson.JSONObject.parseObject(result,OapiV2DepartmentGetResponse.DeptGetResponse.class);
                //OapiV2DepartmentGetResponse.DeptGetResponse deptInfo = dingtalkService.getDeptById(deptIdArr.getLong(i));
                Map<String,Object> mdmMap = new LinkedHashMap<>();
                mdmMap.put("mdmCode",DDDEPTMDMCODE);
                mdmMap.put("optionName","系统管理员");
                Map<String,Object> deptMap =  new LinkedHashMap<>();
                deptMap.put("dept_id",deptInfo.getDeptId());
                deptMap.put("data_id",deptInfo.getDeptId());
                deptMap.put("name",deptInfo.getName());
                deptMap.put("parent_id",deptInfo.getParentId());
                mdmMap.put("mdm_dd_dept",deptMap);
                //保存部门到主数据
                Map<String, String> headerMap = MapBuilder.<String, String>create(true)
                        .put("apiCode", apiCode)//中台接口
                        .put("publicKey",DDPUBLICKEY)//钉钉应用
                        .put("secretKey",DDSECRETKEY)//钉钉应用
                        .put("appId",APPCODE)//中台应用
                        .build();
                String body = HttpRequest.post(URL).addHeaders(headerMap).body(com.alibaba.fastjson.JSONObject.toJSONString(mdmMap)).timeout(60000).execute().body();
                return body;
            }
        }
        return null;
    }
}
