package com.hzya.frame.plugin.kjs.service.impl;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.map.MapBuilder;
import cn.hutool.http.HttpRequest;
import com.dingtalk.api.DefaultDingTalkClient;
import com.dingtalk.api.DingTalkClient;
import com.dingtalk.api.request.OapiV2DepartmentListsubRequest;
import com.dingtalk.api.request.OapiV2UserListRequest;
import com.dingtalk.api.response.OapiV2DepartmentGetResponse;
import com.dingtalk.api.response.OapiV2DepartmentListsubResponse;
import com.dingtalk.api.response.OapiV2UserGetResponse;
import com.dingtalk.api.response.OapiV2UserListResponse;
import com.hzya.frame.dingtalk.enums.OrgEventEnum;
import com.hzya.frame.dingtalk.service.IDingTalkService;
import com.hzya.frame.dingtalk.util.DingTalkAccessToken;
import com.hzya.frame.plugin.kjs.service.GenericEventConsumer;
import com.hzya.frame.plugin.kjs.service.IDingTalkInitService;
import com.taobao.api.ApiException;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;

import java.util.*;

/**
 * @Description 钉钉部门、人员初始化
 * @Author xiangerlin
 * @Date 2024/9/2 16:24
 **/
public class DingTalkInitServiceImpl implements IDingTalkInitService {
    @Value("${dingtalk.appKey:}")
    private String appKey;
    @Value("${dingtalk.appSecret:}")
    private String appSecret;

    @Autowired
    private IDingTalkService dingTalkService;

    @Autowired
    private GenericEventConsumer commonEventConsumer;

    /**
     * 部门初始化，仅初始化部门
     * 这里只会用一次，所以直接调用钉钉原生接口
     * @return
     */
    @Override
    public String deptInit(Long deptId) {
        List<Long> deptIdList = new LinkedList<>();
        List<OapiV2DepartmentListsubResponse.DeptBaseResponse> deptBaseResponseList = new LinkedList<>();
        recursiveDept(deptId, appKey, appSecret,deptIdList,deptBaseResponseList);
        System.out.println(deptIdList.size());
        if (CollectionUtils.isNotEmpty(deptIdList) && deptIdList.size() == deptBaseResponseList.size()){
            for (OapiV2DepartmentListsubResponse.DeptBaseResponse resp : deptBaseResponseList) {
                Map<String,Object> mdmMap = new LinkedHashMap<>();
                mdmMap.put("mdmCode",GenericEventConsumer.DDDEPTMDMCODE);
                mdmMap.put("optionName","系统管理员");
                Map<String,Object> deptMap =  new LinkedHashMap<>();
                deptMap.put("dept_id",resp.getDeptId());
                deptMap.put("data_id",resp.getDeptId());
                deptMap.put("name",resp.getName());
                deptMap.put("parent_id",resp.getParentId());
                mdmMap.put("mdm_dd_dept",deptMap);
                //保存部门到主数据
                Map<String, String> headerMap = MapBuilder.<String, String>create(true)
                        .put("apiCode", GenericEventConsumer.MDMADDCODE)//中台接口
                        .put("publicKey",GenericEventConsumer.DDPUBLICKEY)//钉钉应用
                        .put("secretKey",GenericEventConsumer.DDSECRETKEY)//钉钉应用
                        .put("appId",GenericEventConsumer.APPCODE)//中台应用
                        .build();
                String body = HttpRequest.post(GenericEventConsumer.URL).addHeaders(headerMap).body(com.alibaba.fastjson.JSONObject.toJSONString(mdmMap)).timeout(60000).execute().body();
                System.out.println(body);
            }
        }
        return null;
    }

    private void queryPageUser(OapiV2UserListRequest req,List<OapiV2UserListResponse.ListUserResponse> userList) {
        boolean hasMore = true;
        do {
            hasMore = false;
            OapiV2UserListResponse.PageResult pageResult =  dingTalkService.getUserListByDeptId(req, appKey, appSecret);
            hasMore = pageResult.getHasMore();
            req.setCursor(pageResult.getNextCursor());
            userList.addAll(pageResult.getList());
        }while (hasMore);
    }

    /**
     * 人员初始化
     * @param deptId 部门id
     * @return
     */
    @Override
    public String userInit(Long deptId) {
        List<Long> deptIdList = new LinkedList<>();
        List<OapiV2DepartmentListsubResponse.DeptBaseResponse> deptBaseResponseList = new LinkedList<>();
        recursiveDept(deptId, appKey, appSecret,deptIdList,deptBaseResponseList);
        System.out.println(deptIdList.size());
        if (CollectionUtils.isNotEmpty(deptIdList)){
            for (Long id : deptIdList) {
                List<OapiV2UserListResponse.ListUserResponse> userList = new LinkedList<>();
                //查询部门下人员信息
                OapiV2UserListRequest req = new OapiV2UserListRequest();
                req.setDeptId(id);
                req.setCursor(0L);
                queryPageUser(req,userList);
                if (CollectionUtils.isNotEmpty(userList)){
                    for (OapiV2UserListResponse.ListUserResponse resp : userList) {
                        //查询user详情
                        OapiV2UserGetResponse.UserGetResponse userGetResponse = dingTalkService.getUserById(resp.getUserid(), appKey, appSecret);
                        commonEventConsumer.createMdmUser(OrgEventEnum.USER_MODIFY_ORG.getCode(), userGetResponse);
                    }
                }
            }
        }
        return null;
    }

    private void recursiveDept(Long deptId,String appKey,String appSecret,List<Long> deptIdList,List<OapiV2DepartmentListsubResponse.DeptBaseResponse> deptBaseResponseList){
        if (deptId.longValue() != 1L){
            deptIdList.add(deptId);
        }
        //1、查询部门档案
        List<OapiV2DepartmentListsubResponse.DeptBaseResponse> deptList = dingTalkService.getDeptList(deptId, appKey, appSecret);
        if (CollectionUtils.isNotEmpty(deptList)){
            for (OapiV2DepartmentListsubResponse.DeptBaseResponse deptBaseResponse : deptList) {
                System.out.println(deptBaseResponse.getName());
                deptBaseResponseList.add(deptBaseResponse);
                recursiveDept(deptBaseResponse.getDeptId(),appKey,appSecret,deptIdList,deptBaseResponseList);
            }
        }
    }
}
