package com.hzya.frame.plugin.kjs.util;

import java.util.concurrent.ArrayBlockingQueue;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

/**
 * @Description 用线程消费订阅事件的消息
 * @Author xiangerlin
 * @Date 2024/8/27 15:10
 **/
public class GenericEventThreadPool {
    private static final int MAX_THREADS = 10; // 最大线程数，可以根据需要调整
    private static final int TASK_QUEUE_SIZE = 100; // 任务队列大小，可以根据需要调整

    private final ExecutorService executorService;

    public GenericEventThreadPool() {
        // 创建固定线程数的线程池
        executorService = new ThreadPoolExecutor(
                MAX_THREADS,
                MAX_THREADS,
                0L,
                TimeUnit.MILLISECONDS,
                new ArrayBlockingQueue<>(TASK_QUEUE_SIZE),
                new ThreadPoolExecutor.CallerRunsPolicy());
    }

    //提交任务
    public void executeTask(Runnable task) {
        executorService.execute(task);
    }

    //关闭线程池
    public void close() {
        if (executorService!= null &&!executorService.isShutdown()) {
            executorService.shutdown();
        }
    }

}
